﻿using Lab4.Plugins.Models;
using AutoMapper;
using Lab3.Database.Repository.Interfaces;
using Cop.Borovkov.Var3.Components;
using CustomComponentsVar2;
using ComponentsLibrary;
using System.Windows.Forms;
using ComponentsLibrary.entities;
using ComponentsLibrary.entities.enums;
using Lab3.Database.DTO;
using Lab3.Forms;
using Lab3.Models;
using Lab4.Interfaces;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Configuration;
using Lab3.Extensions;
using Microsoft.Extensions.DependencyInjection;

namespace Lab4.Plugins.Implementations
{
    public class PluginsConvention : IPluginsConvention
    {
        private readonly IMapper _mapper;
        private readonly IStudentRepository _studentRepository;
        private readonly IEducationFormRepository _educationFormRepository;

        private readonly CustomListBox _control;
        private readonly CustomPdfTable _simpleDocumentCreator;
        private readonly CustomExcelTable _tableCreator;
        private readonly ComponentDiagram _chartCreator;

        public PluginsConvention()
        {
            _control = new();
            _chartCreator = new();
            _tableCreator = new();
            _simpleDocumentCreator = new();

            var serviceProvider = CreateServiceProvider();

            _mapper = serviceProvider.GetRequiredService<IMapper>();
            _studentRepository = serviceProvider.GetRequiredService<IStudentRepository>();
            _educationFormRepository = serviceProvider.GetRequiredService<IEducationFormRepository>();
        }

        public string PluginName => "Успеваемость";

        public UserControl GetControl => _control;

        public PluginsConventionElement GetElement
        {
            get
            {
                var filds = _control.Selected.Split();

                Guid id = Guid.Parse(filds[0]);

                return new()
                {
                    Id = id,
                };
            }
        }

        public bool CreateChartDocument(PluginsConventionSaveDocument saveDocument)
        {
            try
            {
                var data = _studentRepository.GetAsync().Result
                    .GroupBy(s => s.EducationForm)
                    .Select(s => new DataLine(
                        s.Key,
                        s.GroupBy(x => x.StartEducation.Year)
                        .Select(x => (year: x.Key.ToString(), count: (double)x.Count()))
                        .ToArray()))
                    .ToList();

                _chartCreator.AddDiagram(
                    new(
                        saveDocument.FileName,
                        "Поступления",
                        "Диаграмма",

                        EnumAreaLegend.Bottom,
                        data
                    )
                );

                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool CreateSimpleDocument(PluginsConventionSaveDocument saveDocument)
        {
            try
            {
                var values = _studentRepository.GetAsync().Result
                        .Select(s => s.StudentSessions
                            .OrderBy(x => x.Number)
                            .Select(x => x.Score.ToString())
                            .ToArray())
                        .ToArray();

                var tables = new string[values.Length, 6];

                for (int i = 0; i < values.Length; ++i)
                {
                    for (int j = 0; j < 6; ++j)
                    {
                        tables[i, j] = values[i][j];
                    }
                }

                _simpleDocumentCreator.SaveToPdf(new()
                {
                    FilePath = saveDocument.FileName,
                    Title = "Сессии",
                    Tables = [tables]
                });

                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool CreateTableDocument(PluginsConventionSaveDocument saveDocument)
        {
            try
            {
                _tableCreator.SaveToExcel<StudentDTO>(new()
                {
                    FilePath = saveDocument.FileName,
                    Title = "Студенты",
                    Headers =
                    [
                        ("Id", nameof(StudentDTO.Id)),
                        ("ФИО", nameof(StudentDTO.Name)),
                        ("Форма обучения", nameof(StudentDTO.EducationForm)),
                        ("Дата поступления", nameof(StudentDTO.StartEducation)),
                    ],
                    HeaderGroups = [new() {
                        GroupHeader = "Образование",
                        FirstHeader = 2,
                        LastHeader = 3,
                    }],
                    Values = _studentRepository.GetAsync().Result
                });

                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool DeleteElement(PluginsConventionElement element)
        {
            try
            {
                _studentRepository
                    .DeleteAsync(element.Id).Wait();

                return true;
            }
            catch
            {
                return false;
            }
        }

        public Form GetForm(PluginsConventionElement? element = null)
            => new CreateForm(_studentRepository, _educationFormRepository, element?.Id);

        public Form GetThesaurus()
            => new CatalogForm(_educationFormRepository);

        public async void ReloadData()
        {
            try
            {
                var students = _mapper.Map<List<StudentViewModel>>(await _studentRepository.GetAsync());

                _control.FillValues(
                    students.Select(s => string.Join(" ",
                        [
                            s.Id,
                        s.Name,
                        s.EducationForm,
                        s.StartEducation.ToLongDateString(),
                        s.SessionMarks,
                        ]
                    ))
                );
            }
            catch
            {

            }
        }

        static IServiceProvider CreateServiceProvider()
        {
            var builder = Host.CreateDefaultBuilder()
                .ConfigureAppConfiguration(c
                    => c.AddJsonFile("appsettings.plugins.json", optional: true, reloadOnChange: true))
                .ConfigureServices((context, services) => {
                    services.ConfigureDAL(context.Configuration);
                    services.AddMapping();
                    services.AddLab3Forms();
                });

            return builder.Build().Services;
        }
    }
}
